{-# OPTIONS_GHC -fno-warn-incomplete-uni-patterns #-}
{-# LANGUAGE TupleSections #-}

-- | This uses ALSA to control the volume.
module XMonad.Util.ALSA ( toggleMute
                        , raiseVolume
                        , lowerVolume
                        -- * Default keybindings
                        , volumeKeys
                        ) where

import           Control.Arrow                (first)
import           Control.Composition
import           Control.Monad
import qualified Data.Map                     as M
import           Graphics.X11.ExtraTypes.XF86
import           Graphics.X11.Types
import           Sound.ALSA.Mixer
import           XMonad

-- | Mute/unmute within the `X` Monad.
toggleMute :: MonadIO m => m ()
toggleMute = liftIO $
    withMixer "default" $ \mixer -> do
        Just control <- getControlByName mixer "Master"
        let Just playbackSwitch = playback $ switch control
        Just sw <- getChannel FrontLeft playbackSwitch
        setChannel FrontLeft playbackSwitch $ not sw

changeVolume :: MonadIO m
             => Channel
             -> Integer -- ^ Percentage
             -> m ()
changeVolume channel i = liftIO $
    withMixer "default" $ \mixer -> do
        Just control <- getControlByName mixer "Master"
        let Just playbackVolume = playback $ volume control
        (min', max') <- getRange playbackVolume
        Just vol <- getChannel channel $ value playbackVolume
        let i' = i * ((max' - min') `div` 100)
        when ((i' > 0 && vol < max') || (i' < 0 && vol > min')) $
            setChannel channel (value playbackVolume) $ vol + i'

-- | Raise volume
raiseVolume :: Integer -> X ()
raiseVolume = axe [ changeVolume FrontLeft, changeVolume FrontRight ]

-- | Lower volume
lowerVolume :: Integer -> X ()
lowerVolume = axe [ changeVolume FrontLeft, changeVolume FrontRight ] . negate

-- | Given your keymaps, add volume control keybindings.
volumeKeys :: M.Map (KeyMask, KeySym) (X ()) -> M.Map (KeyMask, KeySym) (X ())
volumeKeys = M.union mediaKeyMap
    where mediaKeyMap = M.fromList volumeKeyList

volumeKeyList :: [((KeyMask, KeySym), X ())]
volumeKeyList = go <$> [ (xF86XK_AudioMute, toggleMute)
                       , (xF86XK_AudioLowerVolume, lowerVolume 5)
                       , (xF86XK_AudioRaiseVolume, raiseVolume 5)
                       ]
    where go = first (0 ,)
